(function (zreact) {
'use strict';

function getInitDevTools(opt, findDOMNode, findVDom) {
    /**
     * 将zreact的component实例转换为React的
     * @param  component
     */
    function createReactElement(component) {
        var element = {
            key: component._key,
            props: component.props,
            ref: null,
            type: component.constructor,
        };
        return element;
    }
    /**
     * 获取组件名
     * @param  element
     */
    function typeName(element) {
        if (typeof element.type === "function") {
            return element.type.displayName || element.type.name;
        }
        return element.type;
    }
    /**
     * 将component转换实例
     * @param component
     */
    function createReactCompositeComponent(component) {
        var _currentElement = createReactElement(component);
        var node = findDOMNode(component);
        var instance = {
            _currentElement: _currentElement,
            _instance: component,
            forceUpdate: component.forceUpdate && component.forceUpdate.bind(component),
            getName: function getName() {
                return typeName(_currentElement);
            },
            node: node,
            props: component.props,
            setState: component.setState && component.setState.bind(component),
            state: component.state,
        };
        if (component._component) {
            instance._renderedComponent = updateReactComponent(component._component);
        }
        else if (findVDom(component)) {
            instance._renderedComponent = updateReactComponent(findVDom(component));
        }
        return instance;
    }
    /**
     * 将vdom转换为实例
     * @param vdom
     */
    function createReactDOMComponent(vdom) {
        var node = findDOMNode(vdom);
        var childNodes = node && node.nodeType === Node.ELEMENT_NODE ?
            Array.prototype.slice.call(node.childNodes, 0) : [];
        var isText = node && node.nodeType === Node.TEXT_NODE;
        var element;
        if (isText) {
            element = node.textContent;
        }
        else {
            element = {
                props: vdom.props,
                type: node ? node.nodeName.toLowerCase() : "void-node",
            };
        }
        var children = new Array();
        var offset = 0;
        var voidNode = node && node._voidNode;
        var len = childNodes.length;
        var voidLen = -1;
        if (voidNode) {
            for (var key in voidNode) {
                var keynum = +key;
                voidLen = voidLen < keynum ? keynum : voidLen;
            }
            if (voidLen > -1) {
                voidLen += 1;
            }
        }
        if (len > 0) {
            if (voidLen > -1 && voidLen > len) {
                len = voidLen;
            }
            for (var i = 0; i < len; i++) {
                var component = void 0;
                if (voidNode && (i + offset) in voidNode) {
                    component = updateReactComponent(voidNode[(i + offset)].component);
                    offset++;
                    i--;
                    children.push(component);
                    continue;
                }
                var childVDom = findVDom(childNodes[i]);
                if (childVDom && childVDom.component) {
                    component = updateReactComponent(childVDom.component);
                }
                else if (childVDom) {
                    component = updateReactComponent(childVDom);
                }
                if (component) {
                    children.push(component);
                }
            }
        }
        else if (voidNode) {
            for (var key in voidNode) {
                var component = void 0;
                component = updateReactComponent(voidNode[key].component);
                children.push(component);
            }
        }
        // childNodes.forEach(function _(child: any) {
        //     var childVDom = findVDom(child);
        //     if (childVDom) {
        //         let component: IReactComponent;
        //         if (childVDom.component) {
        //             component = updateReactComponent(childVDom.component);
        //         } else {
        //             component = updateReactComponent(childVDom);
        //         }
        //         children.push(component);
        //     }
        // });
        return {
            _currentElement: element,
            _inDevTools: false,
            _renderedChildren: children,
            _stringText: isText ? node.textContent : null,
            node: node,
        };
    }
    var instanceMap = new Map();
    /**
     * 将vdom或component转换为实例
     * @param componentOrVDom
     */
    function updateReactComponent(componentOrVDom) {
        var isVDom = findVDom(componentOrVDom) == null;
        var newInstance = isVDom ? createReactDOMComponent(componentOrVDom) : createReactCompositeComponent(componentOrVDom);
        var base = isVDom ? findDOMNode(componentOrVDom) : componentOrVDom;
        if (instanceMap.has(base)) {
            var inst = instanceMap.get(base);
            Object.assign(inst, newInstance);
            return inst;
        }
        instanceMap.set(base, newInstance);
        return newInstance;
    }
    function nextRootKey(roots) {
        return "." + Object.keys(roots).length;
    }
    function findRoots(node, roots) {
        Array.prototype.forEach.call(node.childNodes, function _(child) {
            var vdom = findVDom(child);
            if (vdom && vdom.component) {
                roots[nextRootKey(roots)] = updateReactComponent(vdom.component);
            }
            else {
                findRoots(child, roots);
            }
        });
    }
    function isRootComponent(component) {
        if (component._parentComponent) {
            return false;
        }
        var base = findDOMNode(component);
        var parentElement = base && base.parentElement;
        var vdom = findVDom(parentElement);
        if (vdom && vdom.props) {
            return false;
        }
        return true;
    }
    function visitNonCompositeChildren(component, visitor) {
        if (component._renderedComponent) {
            if (!component._renderedComponent._component) {
                visitor(component._renderedComponent);
                visitNonCompositeChildren(component._renderedComponent, visitor);
            }
        }
        else if (component._renderedChildren) {
            component._renderedChildren.forEach(function _(child) {
                visitor(child);
                if (!child._component) {
                    visitNonCompositeChildren(child, visitor);
                }
            });
        }
    }
    function createDevToolsBridge(vdom) {
        var ComponentTree = {
            getClosestInstanceFromNode: function getClosestInstanceFromNode(node) {
                while (node && !findVDom(node)) {
                    node = node.parentNode;
                }
                return node ? updateReactComponent(findVDom(node).component) : null;
            },
            getNodeFromInstance: function getNodeFromInstance(instance) {
                return instance.node;
            },
        };
        var roots = {};
        if (vdom && vdom.component) {
            roots[".0"] = updateReactComponent(vdom.component);
        }
        else {
            findRoots(document.body, roots);
        }
        // findRoots(document.body, roots);
        var Mount = {
            _instancesByReactRootID: roots,
            _renderNewRootComponent: function _renderNewRootComponent(arg) { },
        };
        var Reconciler = {
            mountComponent: function mountComponent(arg) { },
            performUpdateIfNecessary: function performUpdateIfNecessary() { },
            receiveComponent: function receiveComponent(arg) { },
            unmountComponent: function unmountComponent(arg) { },
        };
        var componentAdded = function componentAdded_(component) {
            var instance = updateReactComponent(component);
            if (isRootComponent(component)) {
                instance._rootID = nextRootKey(roots);
                roots[instance._rootID] = instance;
                Mount._renderNewRootComponent(instance);
            }
            visitNonCompositeChildren(instance, function _(childInst) {
                childInst._inDevTools = true;
                Reconciler.mountComponent(childInst);
            });
            Reconciler.mountComponent(instance);
        };
        var componentUpdated = function componentUpdated_(component) {
            var prevRenderedChildren = [];
            var node = findDOMNode(component);
            if (!instanceMap.has(node)) {
                componentAdded(component);
                return;
            }
            visitNonCompositeChildren(instanceMap.get(node), function _(childInst) {
                prevRenderedChildren.push(childInst);
            });
            var instance = updateReactComponent(component);
            Reconciler.receiveComponent(instance);
            visitNonCompositeChildren(instance, function _(childInst) {
                if (!childInst._inDevTools) {
                    childInst._inDevTools = true;
                    Reconciler.mountComponent(childInst);
                }
                else {
                    Reconciler.receiveComponent(childInst);
                }
            });
            prevRenderedChildren.forEach(function _(childInst) {
                if (childInst.node && !document.body.contains(childInst.node)) {
                    instanceMap.delete(childInst.node);
                    Reconciler.unmountComponent(childInst);
                }
            });
        };
        var componentRemoved = function componentRemoved_(component) {
            var instance = updateReactComponent(component);
            visitNonCompositeChildren(instance, function _(childInst) {
                instanceMap.delete(childInst.node);
                Reconciler.unmountComponent(childInst);
            });
            Reconciler.unmountComponent(instance);
            instanceMap.delete(component);
            if (instance._rootID) {
                delete roots[instance._rootID];
            }
        };
        return {
            ComponentTree: ComponentTree,
            Mount: Mount,
            Reconciler: Reconciler,
            componentAdded: componentAdded,
            componentRemoved: componentRemoved,
            componentUpdated: componentUpdated,
        };
    }
    return function initDevTools(vdom) {
        if (typeof window.__REACT_DEVTOOLS_GLOBAL_HOOK__ === "undefined") {
            return;
        }
        var bridge = createDevToolsBridge(vdom);
        var nextAfterMount = opt.afterMount;
        opt.afterMount = function (component) {
            bridge.componentAdded(component);
            if (nextAfterMount) {
                nextAfterMount(component);
            }
        };
        var nextAfterUpdate = opt.afterUpdate;
        opt.afterUpdate = function (component) {
            bridge.componentUpdated(component);
            if (nextAfterUpdate) {
                nextAfterUpdate(component);
            }
        };
        var nextBeforeUnmount = opt.beforeUnmount;
        opt.beforeUnmount = function (component) {
            bridge.componentRemoved(component);
            if (nextBeforeUnmount) {
                nextBeforeUnmount(component);
            }
        };
        // Notify devtools about this instance of "React"
        window.__REACT_DEVTOOLS_GLOBAL_HOOK__.inject(bridge);
        return function () {
            opt.afterMount = nextAfterMount;
            opt.afterUpdate = nextAfterUpdate;
            opt.beforeUnmount = nextBeforeUnmount;
        };
    };
}

var initDevTools = getInitDevTools(zreact.options, zreact.findDOMNode, zreact.findVDom);
initDevTools();

}(zreact));
//# sourceMappingURL=devtools-run.js.map
